﻿// Copyright (C) Microsoft Corporation. All Rights Reserved.
// This code released under the terms of the Microsoft Public License
// (Ms-PL, http://opensource.org/licenses/ms-pl.html).

using System;
using System.Diagnostics.CodeAnalysis;
using System.Windows;
#if WINDOWS_PHONE
using Microsoft.Phone.Controls;
#else
using System.Windows.Controls;
#endif

namespace Delay
{
    /// <summary>
    /// Useful extensions for the WebBrowser class.
    /// </summary>
    public static class WebBrowserExtensions
    {
        /// <summary>
        /// Gets the value of the StringSource attached dependency property.
        /// </summary>
        /// <param name="browser">WebBrowser instance for which to get the value.</param>
        /// <returns>Value of the property.</returns>
        [SuppressMessage("Microsoft.Design", "CA1011:ConsiderPassingBaseTypesAsParameters", Justification = "Method applies only to WebBrowser instances.")]
        public static string GetStringSource(WebBrowser browser)
        {
            if (null == browser)
            {
                throw new ArgumentNullException("browser");
            }
            return (string)browser.GetValue(StringSourceProperty);
        }

        /// <summary>
        /// Sets the value of the StringSource attached dependency property.
        /// </summary>
        /// <param name="browser">WebBrowser instance for which to set the value.</param>
        /// <param name="value">Value to set.</param>
        [SuppressMessage("Microsoft.Design", "CA1011:ConsiderPassingBaseTypesAsParameters", Justification = "Method applies only to WebBrowser instances.")]
        public static void SetStringSource(WebBrowser browser, string value)
        {
            if (null == browser)
            {
                throw new ArgumentNullException("browser");
            }
            browser.SetValue(StringSourceProperty, value);
        }

        /// <summary>
        /// Defines the StringSource attached dependency property for the WebBrowser class.
        /// </summary>
        public static readonly DependencyProperty StringSourceProperty = DependencyProperty.RegisterAttached(
            "StringSource", typeof(string), typeof(WebBrowserExtensions), new PropertyMetadata(OnStringSourcePropertyChanged));

        /// <summary>
        /// Handles changes to the StringSource attached dependency property.
        /// </summary>
        /// <param name="o">Instance for which the property changed.</param>
        /// <param name="e">Property change information.</param>
        [SuppressMessage("Microsoft.Naming", "CA2204:Literals should be spelled correctly", MessageId = "WebBrowser", Justification = "Class name.")]
        [SuppressMessage("Microsoft.Naming", "CA2204:Literals should be spelled correctly", MessageId = "StringSource", Justification = "Property name.")]
        private static void OnStringSourcePropertyChanged(DependencyObject o, DependencyPropertyChangedEventArgs e)
        {
            var browser = o as WebBrowser;
            if (null == browser)
            {
                throw new NotSupportedException("The StringSource attached dependency property is only valid for WebBrowser instances.");
            }

            // Determine new HTML content
            var newString = e.NewValue as string;
            var newHtml = string.IsNullOrEmpty(newString) ? "<html></html>" : newString;

#if WINDOWS_PHONE
            try
            {
#endif
                // Set new content
                browser.NavigateToString(newHtml);
#if WINDOWS_PHONE
            }
            catch (InvalidOperationException)
            {
                // WebBrowser on Windows Phone cannot be manipulated outside the visual tree
                // Attempt to re-set content once it enters the visual tree
                RoutedEventHandler browserLoaded = null;
                browserLoaded = delegate(object sender, RoutedEventArgs args)
                {
                    browser.Loaded -= browserLoaded;
                    browser.NavigateToString(newHtml);
                };
                browser.Loaded += browserLoaded;
            }
#endif
        }
    }
}
